import * as TaskManager from 'expo-task-manager';
import * as Notifications from 'expo-notifications';
import { Platform } from 'react-native';
import { API_URL, APP_VERSION } from 'src/constants';
import axios from 'axios';
import { API } from 'src/types';
import { storage, StoreType } from 'src/storage';

const BACKGROUND_NOTIFICATION_TASK = 'BACKGROUND-NOTIFICATION-TASK';

TaskManager.defineTask(
  BACKGROUND_NOTIFICATION_TASK,
  async ({ data, error }: { data?: any; error?: any }) => {
    console.log('Background notification task started');
    if (error) {
      return;
    }

    if (data) {
      const token = storage.get('token', StoreType.STRING);

      const { notification } = data;

      if (!notification.data?.group_token || !notification.data?.message_id) return;

      try {
        const response = await axios.postForm(
          API_URL + '/' + API.GROUP_MESSAGES_RECEIVED,
          {
            token,
            group_token: notification.data.group_token,
            messages_id: [notification.data.message_id]
          },
          {
            headers: {
              Platform: Platform.OS,
              'App-Version': APP_VERSION
            }
          }
        );
      } catch (err) {
        console.error('Error sending notification data:', err);
      }
    }
  }
);

export const registerBackgroundNotificationTask = async () => {
  const isRegistered = await TaskManager.isTaskRegisteredAsync(BACKGROUND_NOTIFICATION_TASK);

  if (!isRegistered) {
    await Notifications.registerTaskAsync(BACKGROUND_NOTIFICATION_TASK);
  }
};

export const unregisterBackgroundNotificationTask = async () => {
  const isRegistered = await TaskManager.isTaskRegisteredAsync(BACKGROUND_NOTIFICATION_TASK);

  if (isRegistered) {
    await TaskManager.unregisterTaskAsync(BACKGROUND_NOTIFICATION_TASK);
  }
};
